﻿#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "csetsubstation.h"
#include "modbus/cmanagemodbus.h"
#include <QDateTime>
#include <QDebug>
#include <QScrollBar>

#pragma execution_character_set("utf-8")

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    m_pSetSub = nullptr;
    m_pModbusSlave = nullptr;
    m_pModbusThread = nullptr;
    initUi();

}

MainWindow::~MainWindow()
{
    delete ui;
}


void MainWindow::on_btnTest_clicked()
{
    if(!m_pSetSub)
    {
        m_pSetSub = new CSetSubstation(this);
    }

    m_pSetSub->show();
}


void MainWindow::on_btnStart_clicked()
{
    ui->btnStart->setEnabled(false);
    ui->btnStop->setEnabled(true);
    initThd();
    insertTable(TypeYxData);
    insertTable(TypeYcData);
}

void MainWindow::on_btnStop_clicked()
{
    ui->btnStart->setEnabled(true);
    ui->btnStop->setEnabled(false);
    m_pModbusSlave->closeModbus();
    m_pModbusSlave->stopThread();

    m_pModbusSlave = nullptr;//已经释放过了
    m_pModbusThread = nullptr;
}

void MainWindow::on_btnConnectServer_clicked()
{

}


void MainWindow::on_btnDisConnectServer_clicked()
{

}

void MainWindow::slotAddLog(bool bServer, bool bRecv, QString msg)
{
    addLog(bServer, bRecv, msg);
}

void MainWindow::initUi()
{
    //ui->btnDisConnectServer->setEnabled(false);
    ui->btnStop->setEnabled(false);
    initTable(ui->tableWidgetYx);
    initTable(ui->tableWidgetYc);
}

void MainWindow::initThd()
{
    if(m_pModbusSlave != nullptr)
    {
        m_pModbusSlave->stopThread();
        QThread::msleep(100);
        delete m_pModbusSlave;
        m_pModbusSlave = nullptr;
    }

    if(m_pModbusThread != nullptr)
    {
        delete m_pModbusThread;
        m_pModbusThread = nullptr;
    }

    QString ip = ui->txtIp->text();
    int nPort = ui->txtPort->text().toInt();
    m_pModbusSlave = new CManageModbus(ip.toStdString(), nPort, true, ui->chkRemoveHead->isChecked());
    S_ModbusConfigInfo config;
    getModbusConfigInfo(config);
    m_pModbusSlave->getConfigInfo(config);
    m_pModbusThread = new QThread;


    m_pModbusSlave->moveToThread(m_pModbusThread);

    bool b = connect(m_pModbusSlave, &CManageModbus::sigAddLog, this, &MainWindow::slotAddLog);
    connect(m_pModbusThread, &QThread::started, m_pModbusSlave, &CManageModbus::doWork);
    connect(m_pModbusThread, &QThread::finished, m_pModbusSlave, &CManageModbus::deleteLater);
    connect(m_pModbusThread, &QThread::finished, m_pModbusThread, &QThread::deleteLater);
    qDebug() << b;
    m_pModbusThread->start();
}

void MainWindow::addLog(bool bServer, bool bRecv, QString msg)
{
    QPlainTextEdit *plainTextEdit = ui->txtLogServer;
    // 设置当前控件为只读
    if(!ui->txtLogServer->isReadOnly())
    {
        ui->txtLogServer->setReadOnly(true);
    }

    // 设置光标到文本末尾
    plainTextEdit->moveCursor(QTextCursor::End,QTextCursor::MoveAnchor);

    if(!bServer)
    {
        plainTextEdit = ui->txtLogServer;
    }
    QString msgType = " <unknown>：";
    if(bRecv)
    {
       msgType = " <Recv>：";
    }
    else
    {
       msgType = " <Send>：";
    }
    QString strTime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss.zzz");
    msg = strTime + msgType + msg + "\n";

    plainTextEdit->insertPlainText(msg);

    // 移动滚动条到底部
    QScrollBar * scrollbar = plainTextEdit->verticalScrollBar();
    if(scrollbar)
    {
        scrollbar->setSliderPosition(scrollbar->maximum());
    }
}

void MainWindow::getModbusConfigInfo(S_ModbusConfigInfo &configInfo)
{
    configInfo.slaveId = ui->txtSlaveIdServer->text().toInt();
    configInfo.yxFuncCode = ui->txtYxFuncIdServer->text().toInt();
    configInfo.ycFuncCode = ui->txtYcFuncIdServer->text().toInt();
    configInfo.yxStartAddr = ui->txtYxAddrServer->text().toInt();
    configInfo.ycStartAddr = ui->txtYcAddrServer->text().toInt();
    configInfo.ymStartAddr = ui->txtYmAddrServer->text().toInt();
    configInfo.yxNum = ui->txtYxNumServer->text().toInt();
    configInfo.ycNum = ui->txtYcNumServer->text().toInt();
    configInfo.ymNum = ui->txtYmNumServer->text().toInt();
}

void MainWindow::initTable(QTableWidget *pTable)
{
    pTable->setRowCount(0);
    pTable->setAlternatingRowColors(true);//隔行变色
    QString strclrHtml = "f0f0f0";
    pTable->setPalette(QPalette(QColor(strclrHtml.toUInt(nullptr, 16))));//隔行变色  颜色 #f0f0f0
    pTable->clear();
    pTable->horizontalHeader()->setVisible(true);
    //pTable->setEditTriggers(QAbstractItemView::DoubleClicked);//双击进入编辑
    //pTable->setSelectionBehavior(QAbstractItemView::SelectRows);//选择整行
    pTable->setSelectionMode(QAbstractItemView::SingleSelection);//只能选择一行
    pTable->setColumnCount(3);//设置列数
//    pTable->setColumnWidth(0, 70);//
//    pTable->setColumnWidth(1, 340);//
//    pTable->setColumnWidth(2, 180);//
//    pTable->setColumnWidth(3, 100);//

    //设置表头动态
    QStringList header;
    header << tr("点地址") << tr("点名称") << tr("码值（Hex）") ;
    pTable->setHorizontalHeaderLabels(header);
    pTable->setSelectionMode(QAbstractItemView::SingleSelection);
    pTable->verticalHeader()->setVisible(false);
    pTable->horizontalHeader()->setMinimumHeight(40);
    pTable->horizontalHeader()->setStretchLastSection(true);
    QHeaderView* headerView = pTable->horizontalHeader();
    headerView->setDefaultAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
}

void MainWindow::insertTable(int type)
{
    switch (type) {
    case TypeYxData:
    {
        ui->tableWidgetYx->setRowCount(0);
        int nStartAddr = ui->txtYxAddrServer->text().toInt();
        QStringList listLine;
        int j = nStartAddr;
        int k = 0;
        for (int i = 0; i < ui->txtYxNumServer->text().toInt(); ++i) {
            listLine.clear();

            listLine << QString("%1_%2").arg(j).arg(k) << QString("遥信%1").arg(i + 1) << QString("0x%1").arg(0, 2, 16, QLatin1Char('0'));
            k++;
            if(k >15)
            {
                j++;
                k = 0;
            }
            insertTableRow(type, ui->tableWidgetYx, listLine);
        }
    }
        break;
    case TypeYcData:
    {
        ui->tableWidgetYc->setRowCount(0);
        int nStartAddr = ui->txtYcAddrServer->text().toInt();
        QStringList listLine;
        int j = nStartAddr;
        for (int i = 0; i < ui->txtYcNumServer->text().toInt(); ++i) {
            listLine.clear();

            listLine << QString("%1").arg(j) << QString("遥测%1").arg(i + 1) << QString("0x%1").arg(0, 4, 16, QLatin1Char('0'));
            j++;

            insertTableRow(type, ui->tableWidgetYc, listLine);
        }
    }
    default:
        break;
    }
}

void MainWindow::insertTableRow(int type, QTableWidget *pTable, const QStringList& listInfo)
{
    int nRowCount = pTable->rowCount();
    pTable->setRowCount(nRowCount + 1);//设置行数
    pTable->setRowHeight(nRowCount, 40);//设置行高

    for(int i = 0; i < listInfo.size(); i++)//消息类型和内容添加
    {
        QTableWidgetItem* item = new QTableWidgetItem(listInfo.at(i));
        if(type == TypeYxData)//遥信表格不可编辑
        {
            item->setFlags(item->flags() & ~Qt::ItemIsEditable);
        }
        else if(type == TypeYcData && i != 2)
        {
            item->setFlags(item->flags() & ~Qt::ItemIsEditable);
        }
        item->setTextAlignment(Qt::AlignCenter);
        pTable->setItem(nRowCount, i, item);
    }
}

void MainWindow::getTableData(QTableWidget *pTable, uint8_t *pData, int &nDataLen)
{
    nDataLen = pTable->rowCount();
    bool b;
    for(int i = 0; i < nDataLen; i++)
    {
        QTableWidgetItem* item = pTable->item(i, 2);
        pData[i] = item->text().toInt(&b, 16);
    }
}

void MainWindow::getTableData(QTableWidget *pTable, uint16_t *pData, int &nDataLen)
{
    nDataLen = pTable->rowCount();
    bool b;
    for(int i = 0; i < nDataLen; i++)
    {
        QTableWidgetItem* item = pTable->item(i, 2);
        pData[i] = item->text().toInt(&b, 16);
    }
}

void MainWindow::on_tableWidgetYx_cellDoubleClicked(int row, int column)
{
    if(column != 2)
    {
        return;
    }
    QTableWidgetItem *item = ui->tableWidgetYx->item(row, column);
    if(item->text().compare("0x00") == 0)
    {
        item->setText("0x01");
    }
    else if(item->text().compare("0x01") == 0)
    {
        item->setText("0x00");
    }
    else
    {
        item->setText("0x00");
    }

    int nLen = 0;
    uint8_t *pData = new uint8_t[60000];
    memset(pData, 0, 60000);
    getTableData(ui->tableWidgetYx, pData, nLen);
    if(m_pModbusSlave != nullptr)
    {
        m_pModbusSlave->getTableData(TypeYxData, pData, nLen);
    }
    if(pData != nullptr)
    {
        delete[] pData;
        pData = nullptr;
    }
}


void MainWindow::on_btnClear_clicked()
{
    ui->txtLogServer->clear();
}


void MainWindow::on_tableWidgetYc_cellChanged(int row, int column)
{
    if(column != 2)
    {
        return;
    }
    QTableWidgetItem *item = ui->tableWidgetYc->item(row, column);

    int nLen = 0;
    uint16_t *pData = new uint16_t[60000];
    memset(pData, 0, 60000);
    getTableData(ui->tableWidgetYc, pData, nLen);

    for (int var = 0; var < nLen; ++var) {
        qDebug() << QString("---%1").arg(pData[var], 4, 16, QLatin1Char('0'));
    }

    if(m_pModbusSlave != nullptr)
    {
        m_pModbusSlave->getTableData(TypeYcData, pData, nLen);
    }
    if(pData != nullptr)
    {
        delete[] pData;
        pData = nullptr;
    }
    bool b = false;
    QString str = QString("0x%1").arg(item->text().toInt(&b, 16), 4, 16, QLatin1Char('0'));
    item->setText(str);
    qDebug() << "on_tableWidgetYc_cellChanged" << "nLen = " << nLen;
}

