﻿#ifndef CMANAGEMODBUS_H
#define CMANAGEMODBUS_H
#include <iostream>
#include <thread>
#include <stdlib.h>
#include <iostream>
#include <mutex>
#include <string>
#include <QObject>
#include <QThread>

/*如果是windows平台则要加载相应的静态库和头文件*/
#ifdef _WIN32
#define _WINSOCK_DEPRECATED_NO_WARNINGS
#include <winsock2.h>
#include <windows.h>
#include "libmodbus/modbus.h"
#pragma comment(lib, "Ws2_32.lib")
//#pragma comment/*(lib, "modbus.lib")*/
/*linux平台*/
#else
#include <modbus/modbus.h>
#include <unistd.h>
#include <error.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/select.h>
#endif

using namespace std;

//#define  MAX_POINT  50000

//coil status 线圈寄存器 使用功能码：01H(读)；05H（写单个位）；0FH(写多个位)
//input status 离散输入寄存器 ：02H
//holding register 保持寄存器 :03H(读)；06H（写单个字节）；0FH(写多个字节)
//input register 输入寄存器：04H

enum DataType
{
    TypeYxData = 0,
    TypeYcData,
    TypeYtData,
    TypeYkData,
    TypeYmData,
    TypeUnknown
};

#pragma pack(1)
//配置信息
struct S_ModbusConfigInfo
{
    int slaveId;//子站地址
    int yxFuncCode;//遥信功能码 规定为02
    int ycFuncCode;//遥测功能码 规定为03
    int yxStartAddr;//遥信首地址
    int ycStartAddr;//遥测首地址
    int ymStartAddr;//遥脉首地址
    int yxNum;//遥信数量
    int ycNum;//遥测数量
    int ymNum;//遥脉数量
    S_ModbusConfigInfo()
    {
        yxFuncCode = 0x02;
        ycFuncCode = 0x03;
    }
};
#pragma pack()

Q_DECLARE_METATYPE(S_ModbusConfigInfo) //注册结构体

class CManageModbus : public QObject
{
    Q_OBJECT

public:
    explicit CManageModbus(string host="0.0.0.0", uint16_t port=502, bool bServer = true, bool bRemoveHead = true, QObject* parent = nullptr);//bServer 启动的是否是服务器
    ~CManageModbus();

public:
    void recieveMessages();//收取信息
    bool modbus_set_slave_id(int id);//设置设备识别码id
    bool initModbus(std::string Host_Ip, int port, bool debugging);//初始化modbus
    
    uint8_t getTab_Input_Bits(int NumBit);
    bool setTab_Input_Bits(int NumBit, uint8_t Value);//向首地址写入 寄存器写入

    uint16_t getHoldingRegisterValue(int registerNumber);
    float getHoldingRegisterFloatValue(int registerStartaddress);
    
    bool setHoldingRegisterValue(int registerNumber, uint16_t Value);
    bool setHoldingRegisterValue(int registerNumber, float Value);

    bool setInputRegisterValue(int registerNumber, uint16_t Value);
    bool setInputRegisterValue(int registerNumber, float Value);
    void closeModbus();//关闭modbus
    void stopThread();//退出线程
    void getConfigInfo(S_ModbusConfigInfo &configInfo);//设置界面配置信息,服务器开始时，收到信号更新配置信息
    void getTableData(int nDataType, uint16_t *pData, int nDataLen);
    void getTableData(int nDataType, uint8_t *pData, int nDataLen);//表格数据写入到mapping
signals:
    void sigAddLog(bool bServer, bool bRecv, QString msg);
public slots:
    void doWork();//线程函数


private:
    std::mutex slavemutex;
    int m_errCount{ 0 };
    int m_modbusSocket{ -1 };
    bool m_initialized{ false };
    modbus_t* ctx{ nullptr };//modbus 实例
    modbus_mapping_t* mapping{ nullptr };
    /*Mapping*/
    int m_numBits{ 60000 };//线圈, (数组元素个数)
    int m_numInputBits{ 60000 };//离散输入, (数组元素个数)
    int m_numRegisters{ 60000 };//输入寄存器(数组元素个数)
    int m_numInputRegisters{ 60000 };// 保持寄存器个数(数组元素个数)
    bool m_bStopThread;//控制线程退出
    bool m_bServer;//是否服务器
    bool m_bRemoveHead;//tcp模式去掉协议头（只保留装置地址，和peugeot.exe保持一致）
    S_ModbusConfigInfo m_configInfo;
private:
    void loadFromConfigFile();
    void run();
    QString unchar2QString(unsigned char* pIn, int nLen);//unsingned char 转16进制字符串
    int QStringHex2Char(const QString& strIn, uint8_t *out, int lenSpace);//16禁止字符串转unsigned char,QString中必须是类似"16 05 78","160578"，5必须补为05
    void addMBAPhead(uint8_t *query, int& rc);//接收到无报文头的，加上
    void removeMBAPhead(uint8_t *query, int& rc);//发送时，再去掉报文头
    void buildSendMapping(const uint8_t *req, int req_len);
    void buildYcMapping(uint16_t *pData, int nDataLen);//遥测对应 0x03
    void buildYxMapping(uint8_t *pData, int nDataLen);//目前遥信对应 0x02
    void buildYkMapping(uint8_t *pData, int nDataLen);//向map
    void buildYtMapping(uint8_t *pData, int nDataLen);
    void buildYmMapping(uint8_t *pData, int nDataLen);
    unsigned int ModBusCRC16(unsigned char *data, unsigned int len);
};
/*annotation:
(1)https://www.jianshu.com/p/0ed380fa39eb
(2)typedef struct _modbus_mapping_t
{
    int nb_bits;                //线圈
    int start_bits;
    int nb_input_bits;          //离散输入
    int start_input_bits;
    int nb_input_registers;     //输入寄存器
    int start_input_registers;
    int nb_registers;           //保持寄存器
    int start_registers;
    uint8_t *tab_bits;
    uint8_t *tab_input_bits;
    uint16_t *tab_input_registers;
    uint16_t *tab_registers;
}modbus_mapping_t;*/
#endif // CMANAGEMODBUS_H

